/**
  ******************************************************************************
  * @file    STemWin/STemWin_HelloWorld/Src/main.c
  * @author  MCD Application Team
  * @version V1.1.0
  * @date    31-August-2017
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2017 STMicroelectronics International N.V.
  * All rights reserved.</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other
  *    contributors to this software may be used to endorse or promote products
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under
  *    this license is void and will automatically terminate your rights under
  *    this license.
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "WM.h"

#define TS_IRQ_BACKGROUND

#include "stm32h743i_eval_ts.h"

/** @addtogroup STM32H7xx_HAL_Applications
  * @{
  */

/** @addtogroup STemWin_HelloWorld
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
static void SystemClock_Config(void);
static void Error_Handler(void);
static void MPU_Config(void);
static void CPU_CACHE_Enable(void);

extern void MainTask(void);

extern unsigned char ts_en;

/* UART handler declaration */
UART_HandleTypeDef UartHandle;


/* Private functions ---------------------------------------------------------*/

#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
   set to 'Yes') calls __io_putchar() */
#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */


#include "colors.h"

//********************************************
#include "btn.h"

#define BTN_CNT 1
#define BTN_IMG 0x00000001
#define BTN_RST 0x00000002

btn_area_t btns[BTN_CNT];

uint32_t  btn_state = 0;
uint32_t  btn_state_prev = 0;
uint32_t  btn_active = 0;
uint32_t  btn_press = 0;
uint32_t  btn_release = 0;


//********************************************
#include "sldr.h"
#define SLDR_V_CNT  3
#define SLDR_V_AG  0x00000001
#define SLDR_V_DG  0x00000002
#define SLDR_V_MG  0x00000004

sldr_area_t sldrs_v[SLDR_V_CNT];

uint32_t  sldr_v_state = 0;

//********************************************
#include "chck.h"
#define CHCK_CNT 1
#define CHCK_0  0x00000001

chck_area_t chcks[CHCK_CNT];

uint32_t  chck_state = 0;
uint32_t  chck_state_prev = 0;
uint32_t  chck_checked = 1;
uint32_t  chck_active = 0;
uint32_t  chck_press = 0;
uint32_t  chck_release = 0;


//********************************************
#define IMG_X_SIZE    256 //213 //182
#define IMG_Y_SIZE    204 //170 //146
#define IMG_PIX_BYTES 3// uint8_t img[IMG_SIZE * IMG_PIX_BYTES];


#define MSG_MAX_LEN ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) + 6)
#define MSG_START '#'
#define MSG_STOP  '*'

uint8_t   msg_ch;
unsigned char  msg[MSG_MAX_LEN];
// typedef uint8_t msg_t[MSG_MAX_LEN];

// msg_t *msg = (msg_t *)SDRAM_USR_ADDR;

// static uint8_t  *msg = (uint8_t *)SDRAM_USR_ADDR;
// uint16_t  msg_len;
// uint8_t   msg_active;
uint8_t   msg_parse;


GUI_CONST_STORAGE GUI_BITMAP _bm_img= {
  IMG_X_SIZE,                   /* XSize */
  IMG_Y_SIZE,                   /* YSize */
  IMG_X_SIZE * IMG_PIX_BYTES,   /* BytesPerLine */
  IMG_PIX_BYTES * 8,            /* BitsPerPixel */
  msg,                          /* Pointer to picture data */
  NULL,                         /* Pointer to palette */
  GUI_DRAW_BMP24
};


TIM_HandleTypeDef TimHandle;
volatile uint8_t GUI_Initialized = 0;

#ifdef TS_IRQ_BACKGROUND
uint32_t uwPrescalerValue = 0;

void BSP_Background(void);
#endif

void BSP_Pointer_Update(void);

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
  /* This project template calls firstly two functions in order to configure MPU feature
     and to enable the CPU Cache, respectively MPU_Config() and CPU_CACHE_Enable().
     These functions are provided as template implementation that User may integrate
     in his application, to enhance the performance in case of use of AXI interface
     with several masters. */

  /* Configure the MPU attributes as Write Through */
  MPU_Config();

  /* Enable the CPU Cache */
  CPU_CACHE_Enable();

  /* STM32H7xx HAL library initialization:
       - Configure the Systick to generate an interrupt each 1 msec
       - Set NVIC Group Priority to 4
       - Low Level Initialization
     */
  HAL_Init();

  /* Configure the system clock to 400 MHz */
  SystemClock_Config();

  /* Initializes the SDRAM device */
  BSP_SDRAM_Init();

  /* Init the STemWin GUI Library */
  __HAL_RCC_CRC_CLK_ENABLE(); /* Enable the CRC Module */

  GUI_Init();

  /*
   * Initialize touch layer
   */
  BSP_TS_Init(640, 480);


  /***********************************************************/
  /* Put the USART peripheral in the Asynchronous mode (UART Mode) */
  /* UART configured as follows:
      - Word Length = 8 Bits (7 data bit + 1 parity bit) : BE CAREFUL : Program 7 data bits + 1 parity bit in PC HyperTerminal
      - Stop Bit    = One Stop bit
      - Parity      = no parity
      - BaudRate    = 115200 baud
      - Hardware flow control disabled (RTS and CTS signals) */
  UartHandle.Instance        = USARTx;

  UartHandle.Init.BaudRate   = 115200*5;        //230400;  //921600;  //115200;
  UartHandle.Init.WordLength = UART_WORDLENGTH_8B;
  UartHandle.Init.StopBits   = UART_STOPBITS_2;
  UartHandle.Init.Parity     = UART_PARITY_NONE;
  UartHandle.Init.HwFlowCtl  = UART_HWCONTROL_NONE;
  UartHandle.Init.Mode       = UART_MODE_TX_RX;
  UartHandle.Init.OverSampling = UART_OVERSAMPLING_16;

  if(HAL_UART_Init(&UartHandle) != HAL_OK)
  {
    /* Initialization Error */
    Error_Handler();
  }


//   printf("\n\r UART Printf Example: retarget the C library printf function to the UART\n\r");
//   printf("** Test finished successfully. ** \n\r");


#ifdef TS_IRQ_BACKGROUND
  /* Compute the prescaler value to have TIM3 counter clock equal to 10 KHz */
  uwPrescalerValue = (uint32_t) ((SystemCoreClock /2) / 10000) - 1;

  /* Set TIMx instance */
  TimHandle.Instance = TIM3;

//   TimHandle.Init.Period = 500 - 1;
  TimHandle.Init.Period = 650;
  TimHandle.Init.Prescaler = uwPrescalerValue;
  TimHandle.Init.ClockDivision = 0;
  TimHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
  if(HAL_TIM_Base_Init(&TimHandle) != HAL_OK)
  {
    Error_Handler();
  }

  /* Start Channel1 */
  if(HAL_TIM_Base_Start_IT(&TimHandle) != HAL_OK)
  {
    Error_Handler();
  }
#endif

  /* Run the HelloWorld erxample */
  MainTask();

  GUI_Initialized = 1;


  while (1) {

#ifndef TS_IRQ_BACKGROUND
    BSP_Pointer_Update();
#endif

  }

}


/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART1 and Loop until the end of transmission */
  HAL_UART_Transmit(&UartHandle, (uint8_t *)&ch, 1, 0xFFFF);

  return ch;
}


#ifdef TS_IRQ_BACKGROUND
/**
  * @brief  Period elapsed callback in non blocking mode
  * @param  htim: TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
  BSP_Background();
}


/**
  * @brief TIM MSP Initialization
  *        This function configures the hardware resources used in this application:
  *           - Peripheral's clock enable
  *           - Peripheral's GPIO Configuration
  * @param htim: TIM handle pointer
  * @retval None
  */
void HAL_TIM_Base_MspInit(TIM_HandleTypeDef *htim)
{
  /*##-1- Enable peripherals and GPIO Clocks #################################*/
  /* TIMx Peripheral clock enable */
  __HAL_RCC_TIM3_CLK_ENABLE();

  /*##-2- Configure the NVIC for TIMx #########################################*/
  /* Set the TIMx priority */
  HAL_NVIC_SetPriority(TIM3_IRQn, 0, 1);

  /* Enable the TIMx global Interrupt */
  HAL_NVIC_EnableIRQ(TIM3_IRQn);
}


/**
* @brief  BSP_Background.
* @param  None
* @retval None
*/
void BSP_Background(void)
{
  /* Capture input event and updade cursor */
  if(GUI_Initialized == 1)
  {
    BSP_Pointer_Update();
  }
}
#endif


/**
  * @brief  Provide the GUI with current state of the touch screen
  * @param  None
  * @retval None
  */
void BSP_Pointer_Update(void)
{
    TS_StateTypeDef  ts;

    ts.x = 0;
    ts.y = 0;
    ts.TouchDetected = 0;
    BSP_TS_GetState(&ts);

    HAL_StatusTypeDef status;
//     int i;
//     int j;


    btn_state = get_button(&ts, btns, BTN_CNT);

    if ( btn_state != btn_state_prev ) {


        btn_press   = ~btn_active &  btn_state;
        btn_release =  btn_active & ~btn_state;

//         if ( btn_press & BTN_RST ) {
//             btn_active |= BTN_RST;
//             draw_button(&btns[0], BTN_ACTIVE, "Reinit");
//             printf("!RST*\r\n");
//         }
//         if ( btn_release & BTN_RST ) {
//             btn_active &= ~BTN_RST;
//             draw_button(&btns[0], BTN_INACTIVE, "Reinit");
//         }

        if ( btn_press & BTN_IMG ) {
            btn_active |= BTN_IMG;
            draw_button(&btns[0], BTN_ACTIVE, "Preview");
            printf("!GIMG*\r\n");

//             msg_active = 0;
//             msg_len = 0;
            msg_parse = 0;


            // HAL_UART_Receive can receive 2^16
            status = HAL_UART_Receive(&UartHandle, (uint8_t *)msg, 4, 0xFFFF);

            if ( (msg[0] == '#') && (msg[1] == 'I') && (msg[2] == 'M') && (msg[3] == 'G') ) {

//                 status = HAL_UART_Receive(&UartHandle, (uint8_t *)msg, ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2), 0xFFFF);
//                 status = HAL_UART_Receive(&UartHandle, (uint8_t *)(msg+(((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2))), ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 2) + 1, 0xFFFF);

                status = HAL_UART_Receive(&UartHandle, (uint8_t *)msg, (( IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 3), 0xFFFF);
                status = HAL_UART_Receive(&UartHandle, (uint8_t *)(msg+(((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 3))), ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 3), 0xFFFF);
                status = HAL_UART_Receive(&UartHandle, (uint8_t *)(msg+(2*((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 3))), ((IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES) / 3) + 1, 0xFFFF);

//                 for (int i = 0; i < IMG_X_SIZE * IMG_Y_SIZE * IMG_PIX_BYTES; i++) {
//                     status = HAL_UART_Receive(&UartHandle, (uint8_t *)&msg[i], 1, 0xFFFF);
//                 }


                if (status == HAL_OK) {
                    msg_parse = 1;
                }
            }



            // parse received message
            if ( msg_parse ) {

                msg_parse = 0;
                GUI_DrawBitmap(&_bm_img, 345+3, 170+3);

            }

        }
        if ( btn_release & BTN_IMG ) {
            btn_active &= ~BTN_IMG;
            draw_button(&btns[0], BTN_INACTIVE, "Preview");
        }


        btn_state_prev = btn_state;
    }


    sldr_v_state = get_slider_v(&ts, sldrs_v, SLDR_V_CNT);

    if ( sldr_v_state ) {

        if ( ( sldr_v_state & SLDR_V_AG ) && (sldrs_v[0].pos != sldrs_v[0].pos_prev) ) {
            draw_slider_v(&sldrs_v[0]);
            printf("!SAG%d*\r\n", sldrs_v[0].pos);
            sldrs_v[0].pos_prev = sldrs_v[0].pos;
        }

        if ( ( sldr_v_state & SLDR_V_DG ) && (sldrs_v[1].pos != sldrs_v[1].pos_prev) ) {
            draw_slider_v(&sldrs_v[1]);
            printf("!SDG%d*\r\n", sldrs_v[1].pos);
            sldrs_v[1].pos_prev = sldrs_v[1].pos;
        }

        if ( ( sldr_v_state & SLDR_V_MG ) && (sldrs_v[2].pos != sldrs_v[2].pos_prev) ) {
            if (sldrs_v[2].pos == 0) sldrs_v[2].pos = 1; // fix the minima value
            draw_slider_v(&sldrs_v[2]);
            printf("!SMG%d*\r\n", sldrs_v[2].pos);
            sldrs_v[2].pos_prev = sldrs_v[2].pos;
        }
    }


    chck_state = get_checker(&ts, chcks, CHCK_CNT);

    if ( chck_state != chck_state_prev ) {

        chck_press   = ~chck_active &  chck_state;
        chck_release =  chck_active & ~chck_state;

//         GUI_SetBkColor(UTIA_COLOR_GRAY_L);

        if ( chck_press & CHCK_0 ) {
            GUI_SetBkColor(UTIA_COLOR_GRAY_L);
            if ( chck_checked & CHCK_0 ) {
                chck_checked &= ~CHCK_0;
                draw_checker(&chcks[0], CHCK_INACTIVE);
                GUI_SetColor(GUI_GRAY);
                GUI_DispStringAt("Filter", 410, 400);
                GUI_SetColor(UTIA_COLOR_BLUE2);
                GUI_DispStringAt("Camera", 410, 425);
                printf("!SOC*\r\n");
            } else {
                chck_checked |= CHCK_0;
                draw_checker(&chcks[0], CHCK_ACTIVE);
                GUI_SetColor(UTIA_COLOR_BLUE2);
                GUI_DispStringAt("Filter", 410, 400);
                GUI_SetColor(GUI_GRAY);
                GUI_DispStringAt("Camera", 410, 425);
                printf("!SOF*\r\n");
            }

        }

        chck_state_prev = chck_state;
    }


}


/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow :
  *            System Clock source            = PLL (HSE)
  *            SYSCLK(Hz)                     = 400000000 (CPU Clock)
  *            HCLK(Hz)                       = 200000000 (AXI and AHBs Clock)
  *            AHB Prescaler                  = 2
  *            D1 APB3 Prescaler              = 2 (APB3 Clock  100MHz)
  *            D2 APB1 Prescaler              = 2 (APB1 Clock  100MHz)
  *            D2 APB2 Prescaler              = 2 (APB2 Clock  100MHz)
  *            D3 APB4 Prescaler              = 2 (APB4 Clock  100MHz)
  *            HSE Frequency(Hz)              = 25000000
  *            PLL_M                          = 5
  *            PLL_N                          = 160
  *            PLL_P                          = 2
  *            PLL_Q                          = 4
  *            PLL_R                          = 2
  *            VDD(V)                         = 3.3
  *            Flash Latency(WS)              = 4
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;
  HAL_StatusTypeDef ret = HAL_OK;

  /*!< Supply configuration update enable */
  MODIFY_REG(PWR->CR3, PWR_CR3_SCUEN, 0);

  /* The voltage scaling allows optimizing the power consumption when the device is
     clocked below the maximum system frequency, to update the voltage scaling value
     regarding system frequency refer to product datasheet.  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  while ((PWR->D3CR & (PWR_D3CR_VOSRDY)) != PWR_D3CR_VOSRDY) {}

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_OFF;
  RCC_OscInitStruct.CSIState = RCC_CSI_OFF;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;

  RCC_OscInitStruct.PLL.PLLM = 5;
  RCC_OscInitStruct.PLL.PLLN = 160;
  RCC_OscInitStruct.PLL.PLLP = 2;
  RCC_OscInitStruct.PLL.PLLR = 2;
  RCC_OscInitStruct.PLL.PLLQ = 4;

  RCC_OscInitStruct.PLL.PLLVCOSEL = RCC_PLL1VCOWIDE;
  RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1VCIRANGE_2;
  ret = HAL_RCC_OscConfig(&RCC_OscInitStruct);
  if(ret != HAL_OK)
  {
    Error_Handler();
  }

/* Select PLL as system clock source and configure  bus clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_D1PCLK1 | RCC_CLOCKTYPE_PCLK1 | \
                                 RCC_CLOCKTYPE_PCLK2  | RCC_CLOCKTYPE_D3PCLK1);

  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.SYSCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB3CLKDivider = RCC_APB3_DIV2;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_APB1_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_APB2_DIV2;
  RCC_ClkInitStruct.APB4CLKDivider = RCC_APB4_DIV2;
  ret = HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4);
  if(ret != HAL_OK)
  {
    Error_Handler();
  }

  /*activate CSI clock mondatory for I/O Compensation Cell*/
  __HAL_RCC_CSI_ENABLE() ;

  /* Enable SYSCFG clock mondatory for I/O Compensation Cell */
  __HAL_RCC_SYSCFG_CLK_ENABLE() ;

  /* Enables the I/O Compensation Cell */
  HAL_EnableCompensationCell();
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
static void Error_Handler(void)
{
  /* User may add here some code to deal with this error */
  while(1)
  {
  }
}

/**
  * @brief  Configure the MPU attributes as Write Through for Internal D1SRAM.
  * @note   The Base Address is 0x24000000 since this memory interface is the AXI.
  *         The Configured Region Size is 512KB because same as D1SRAM size.
  * @param  None
  * @retval None
  */
static void MPU_Config(void)
{
  MPU_Region_InitTypeDef MPU_InitStruct;

  /* Disable the MPU */
  HAL_MPU_Disable();

  /* Configure the MPU attributes as WT for SRAM */
  MPU_InitStruct.Enable = MPU_REGION_ENABLE;
  MPU_InitStruct.BaseAddress = 0x24000000;
  MPU_InitStruct.Size = MPU_REGION_SIZE_512KB;
  MPU_InitStruct.AccessPermission = MPU_REGION_FULL_ACCESS;
  MPU_InitStruct.IsBufferable = MPU_ACCESS_NOT_BUFFERABLE;
  MPU_InitStruct.IsCacheable = MPU_ACCESS_CACHEABLE;
  MPU_InitStruct.IsShareable = MPU_ACCESS_SHAREABLE;
  MPU_InitStruct.Number = MPU_REGION_NUMBER0;
  MPU_InitStruct.TypeExtField = MPU_TEX_LEVEL0;
  MPU_InitStruct.SubRegionDisable = 0x00;
  MPU_InitStruct.DisableExec = MPU_INSTRUCTION_ACCESS_ENABLE;

  HAL_MPU_ConfigRegion(&MPU_InitStruct);

  /* Configure the MPU attributes as WT for SDRAM */
  MPU_InitStruct.Enable           = MPU_REGION_ENABLE;
  MPU_InitStruct.BaseAddress      = 0xD0000000;
  MPU_InitStruct.Size             = MPU_REGION_SIZE_32MB;
  MPU_InitStruct.AccessPermission = MPU_REGION_FULL_ACCESS;
  MPU_InitStruct.IsBufferable     = MPU_ACCESS_NOT_BUFFERABLE;
  MPU_InitStruct.IsCacheable      = MPU_ACCESS_CACHEABLE;
  MPU_InitStruct.IsShareable      = MPU_ACCESS_SHAREABLE;
  MPU_InitStruct.Number           = MPU_REGION_NUMBER1;
  MPU_InitStruct.TypeExtField     = MPU_TEX_LEVEL0;
  MPU_InitStruct.SubRegionDisable = 0x00;
  MPU_InitStruct.DisableExec      = MPU_INSTRUCTION_ACCESS_DISABLE;

  HAL_MPU_ConfigRegion(&MPU_InitStruct);

  /* Enable the MPU */
  HAL_MPU_Enable(MPU_PRIVILEGED_DEFAULT);
}

/**
  * @brief  CPU L1-Cache enable.
  * @param  None
  * @retval None
  */
static void CPU_CACHE_Enable(void)
{
  /* Enable I-Cache */
  SCB_EnableICache();

  /* Enable D-Cache */
  SCB_EnableDCache();
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
