/*********************************************************************
*                SEGGER MICROCONTROLLER SYSTEME GmbH                 *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2017  SEGGER Microcontroller Systeme GmbH        *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

***** emWin - Graphical user interface for embedded applications *****
emWin is protected by international copyright laws.   Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with a license and should not be re-
distributed in any way. We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : BASIC_HelloWorld.c
Purpose     : Simple demo drawing "Hello world"
----------------------------------------------------------------------
*/
#include <stddef.h>

#include "GUI.h"

#include "colors.h"
#include "btn.h"
#include "sldr.h"
#include "chck.h"


/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

extern GUI_CONST_STORAGE unsigned char _ac_logo_sp[];

GUI_CONST_STORAGE GUI_BITMAP _bm_logo_sp= {
  327,                          /* XSize */
  57,                           /* YSize */
  981,                          /* BytesPerLine */
  24,                           /* BitsPerPixel */
  (unsigned char *)_ac_logo_sp, /* Pointer to picture data */
  NULL,                         /* Pointer to palette */
  GUI_DRAW_BMP24
};



// button areas
extern btn_area_t btns[];
extern chck_area_t chcks[];
extern sldr_area_t sldrs_v[];


unsigned char ts_en = 0;


/*********************************************************************
*
*       MainTask
*/
void MainTask(void) {
  GUI_SetLayerVisEx (1, 0);
  GUI_SetBkColor(GUI_BLACK);
  GUI_Clear();
//   GUI_SetColor(GUI_WHITE);
//   GUI_SetFont(&GUI_Font32_1);
//   GUI_DispStringAt("Hello world!", (LCD_GetXSize()-100)/2, (LCD_GetYSize()-20)/2); 


  GUI_SetBkColor(UTIA_COLOR_GRAY_L);
  
  GUI_SetColor(GUI_WHITE);
  GUI_FillRect(0, 0, 640, 480);

  GUI_DrawBitmap(&_bm_logo_sp, 10, 10);
  
  GUI_SetColor(UTIA_COLOR_BLUE1);
  GUI_FillRect(10, 85, 630, 120);
  GUI_SetColor(UTIA_COLOR_GRAY_L);
  GUI_FillRect(10, 120, 630, 470);
    
  GUI_SetColor(UTIA_COLOR_BLUE2);
//   GUI_DrawRoundedFrame(345, 170, 558+5, 340+5, 1, 3);
  GUI_DrawRoundedFrame(345, 170, 601+5, 374+5, 1, 3);
  GUI_SetColor(GUI_BLACK);
//   GUI_FillRect(345+3, 170+3, 558+2, 340+2);
  GUI_FillRect(345+3, 170+3, 601+2, 374+2);

  GUI_SetColor(UTIA_COLOR_BLUE2);
  GUI_SetBkColor(UTIA_COLOR_BLUE1);
  GUI_SetFont(&GUI_Font24B_ASCII);
//   GUI_DispStringAt("Camera Python 1300 Configuration", 15, 90);
  GUI_DispStringAt("Optical Flow Demonstrator Configuration", 15, 90);

  GUI_SetBkColor(UTIA_COLOR_GRAY_L);

  
  GUI_DispStringAt("Camera Preview", 346, 140);

  // analog gain
  GUI_DispStringHCenterAt("Analog\r\nGain", 65, 140);
  sldrs_v[0].x0 = 50;
  sldrs_v[0].x1 = 80;
  sldrs_v[0].y0 = 190;
  sldrs_v[0].y1 = 420;
  sldrs_v[0].pos = 0;
  sldrs_v[0].pos_prev = 0;
  sldrs_v[0].scale = 10;
  draw_slider_v(&sldrs_v[0]);

  // Digital gain
  GUI_SetFont(&GUI_Font24B_ASCII);
//   GUI_DispStringHCenterAt("Digital\r\nGain", 195, 140);
  GUI_DispStringHCenterAt("Digital\r\nGain", 165, 140);
  sldrs_v[1].x0 = 150;  //180;
  sldrs_v[1].x1 = 180;  //210;
  sldrs_v[1].y0 = 190;
  sldrs_v[1].y1 = 420;
  sldrs_v[1].pos = 128;
  sldrs_v[1].pos_prev = 128;
  sldrs_v[1].scale = 1024;  //4095;
  draw_slider_v(&sldrs_v[1]);

  // Optical flow magnitude limit
  GUI_SetFont(&GUI_Font24B_ASCII);
  GUI_DispStringHCenterAt("Magnitude\r\nLimit", 265, 140);
  sldrs_v[2].x0 = 250;
  sldrs_v[2].x1 = 280;
  sldrs_v[2].y0 = 190;
  sldrs_v[2].y1 = 420;
  sldrs_v[2].pos = 10;
  sldrs_v[2].pos_prev = 10;
  sldrs_v[2].scale = 32;
  draw_slider_v(&sldrs_v[2]);

  
  // button Reinit
//   btns[0].x0 = 463+48;
//   btns[0].x1 = 563+48;
//   btns[0].y0 = 395;
//   btns[0].y1 = 445;
//   draw_button(&btns[0], BTN_INACTIVE, "Reinit");

//   button 1
//   btns[0].x0 = 342+60;
//   btns[0].x1 = 442+60;
//   btns[0].y0 = 395;
//   btns[0].y1 = 445;
//   draw_button(&btns[0], BTN_INACTIVE, "Preview");
  btns[0].x0 = 463+48;
  btns[0].x1 = 563+48;
  btns[0].y0 = 395;
  btns[0].y1 = 445;
  draw_button(&btns[0], BTN_INACTIVE, "Preview");


  // filter enable
  chcks[0].x = 342;
  chcks[0].y = 395;
  chcks[0].s = 50;
  draw_checker(&chcks[0], CHCK_ACTIVE);

  GUI_SetBkColor(UTIA_COLOR_GRAY_L);
  GUI_DispStringAt("Filter", 410, 400);
  GUI_SetColor(GUI_GRAY);
  GUI_DispStringAt("Camera", 410, 425);

  //enable touches after gui has been drawn
  ts_en = 1;


//   while(1) {};
}

/*************************** End of file ****************************/
